package edu.ucr.runner;

/*
 * Copyright 2020 Renjie Wu
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import com.timeseries.TimeSeries;
import edu.ucr.impl.DtwUpd;
import edu.ucr.util.SquaredDistance;
import edu.ucr.util.TsvReader;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;

public class PairwiseRunner {
    private static final int[] FASTDTW_RADIUS = { 10, 20, 30, 40 };

    public static void main(String[] args) throws IOException {
        var label = new ArrayList<Integer>();
        var data = new ArrayList<double[]>();
        var dataTS = new ArrayList<TimeSeries>();

        TsvReader.readTsv(args[0], data, dataTS, label);

        try (var outputWriter = new PrintWriter(args[1])) {
            outputWriter.print("timeSeries1,timeSeries2");
            for (var radius : FASTDTW_RADIUS) {
                outputWriter.format(",FastDTW%dDist", radius);
            }
            outputWriter.print(",fullDTWDist");
            outputWriter.println();

            for (int i = 0; i < label.size(); i++) {
                for (int j = i + 1; j < label.size(); j++) {
                    outputWriter.format("%d,%d", label.get(i), label.get(j));
                    for (var radius : FASTDTW_RADIUS) {
                        var fastDtw = com.dtw.FastDTW.getWarpDistBetween(dataTS.get(i), dataTS.get(j), radius, new SquaredDistance());
                        outputWriter.format(",%.6f", fastDtw);
                    }

                    var cDtw = DtwUpd.computeDtw(data.get(i), data.get(j), data.get(i).length); // full dtw
                    outputWriter.format(",%.6f", cDtw);
                    outputWriter.println();
                    outputWriter.flush();
                }
            }
        }
    }
}
